package org.xdoclet.plugin.ejb.qtags;

/**
 * The ejb.bean tag provides information about the EJB. It is the one compulsory tag
 * for all EJBs, however not all parameters are applicable for all types of beans,
 * and some parameters apply differently for different types of beans.
 */
public interface EjbBeanTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * Provides the name of the bean that will be used in the ejb-jar.xml
     * deployment descriptor.
     *
     * @qtags.required
     */
    String getName_();

    /**
     * A description of the bean.  Defaults to the first sentence of the
     * class-level javadoc comment of the bean class.
     *
     */
    String getDescription();

    /**
     * The display name of the bean.
     *
     */
    String getDisplayName();

    /**
     * The small icon for the bean.
     *
     */
    String getSmallIcon();

    /**
     * The large icon for the bean.
     *
     */
    String getLargeIcon();

    /**
     * Set it to "false" if you want the class be excluded from list of EJBs.
     * This is useful for abstract EJBs that other EJBs are to derive from. So
     * you obviously do not want the abstract EJB be wrongly specified in deployment
     * descriptors as a concrete EJB. Default is "true".
     *
     */
    boolean isGenerate();

    /**
     * Provides the JNDI name of the
     * bean that will be used in the
     * vendor specific deployment descriptors.
     *
     */
    String getJndiName();

    /**
     * Provides the JNDI name of the bean
     * that will be used in the vendor
     * specific deployment descriptors.
     * It's the JNDI name for the local EJB.
     *
     */
    String getLocalJndiName();

    /**
     * Indicates what view-type(s) is(are)
     * supported by the bean.
     *
     * @qtags.allowed-value local
     * @qtags.allowed-value remote
     * @qtags.allowed-value both
     */
    String getViewType();

    /**
     * Defines the bean's "type".
     *
     * @qtags.required
     * @qtags.allowed-value CMP
     * @qtags.allowed-value BMP
     * @qtags.allowed-value Stateless
     * @qtags.allowed-value Stateful
     */
    String getType();

    /**
     * Defines the bean's transaction type.
     *
     * @qtags.allowed-value Container
     * @qtags.allowed-value Bean
     */
    String getTransactionType();

    /**
     * Defines the entity bean's reentrancy.
     *
     */
    boolean isReentrant();

    /**
     * Defines the version of the spec
     * the bean is for CMP.
     *
     * @qtags.allowed-value 1.x
     * @qtags.allowed-value 2.x
     */
    String getCmpVersion();

    /**
     * Defines the abstract schema name for the bean.
     *
     */
    String getSchema();

    /**
     * Defines the primary key field for
     * the bean as per spec.
     *
     */
    String getPrimkeyField();

    /**
     * If true, then generate a public int
     * version attribute in the auto-generated
     * concrete class, to form a optimistic
     * locking mechanism. The value of version
     * attribute is incremented in ejbStore.
     * The default is false.
     * Note: It's applicable to both CMP and BMP, but many containers have optimistic locking
     * capabilities built-in, so you don't need to worry about it locking issues.
     *
     */
    boolean isUseSoftLocking();

    /**
     * Defines an optional message selector for MDB.
     *
     */
    String getMessageSelector();

    /**
     * Defines the acknowledge mode for MDB.
     *
     * @qtags.allowed-value Auto-acknowledge
     * @qtags.allowed-value Dups-ok-acknowledge
     */
    String getAcknowledgeMode();

    /**
     * Defines the type of destination for MDB.
     *
     * @qtags.allowed-value javax.jms.Queue
     * @qtags.allowed-value javax.jms.Topic
     */
    String getDestinationType();

    /**
     * Defines the durability of messages
     * for MDB as per EJB 2.0 spec
     *
     * @qtags.allowed-value Durable
     * @qtags.allowed-value NonDurable
     */
    String getSubscriptionDurability();

    /**
     * Defines the remote business interface
     * for the bean. If present, ejb.interface-method
     * tags are not required, instead the remote interface will extend the business interface.
     * The business interface must declare all methods as throwing RemoteException as per the
     * business interface pattern.
     *
     */
    String getRemoteBusinessInterface();

    /**
     * Defines the local business interface for the bean. Similar to remote-business-interface,
     * but the interface does not need to declare RemoteException in the throws clause.
     *
     */
    String getLocalBusinessInterface();

}
