/*
 * Copyright (c) 2003-2004
 * XDoclet Team
 * All rights reserved.
 */
package org.xdoclet.plugin.hibernate;

import com.thoughtworks.qdox.model.AbstractJavaEntity;
import com.thoughtworks.qdox.model.BeanProperty;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaField;
import com.thoughtworks.qdox.model.JavaMethod;

import org.generama.JellyTemplateEngine;
import org.generama.MetadataProvider;
import org.generama.Plugin;
import org.generama.WriterMapper;
import org.xdoclet.plugin.hibernate.qtags.TagLibrary;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * plugin producing hibernate mapping declarations
 *
 * @author Konstantin Pribluda
 * @version $Revision: 1.23 $
 */
public class HibernateMappingPlugin extends Plugin {


    static Map tagDispatch = new HashMap();
    private static final String PACKAGE_PREFIX = "org/xdoclet/plugin/hibernate/";
    private static final String TAG_PREFIX = "hibernate.";

    // list holding tag names defining property existence
    private final static List PROPERTY_TAGS = new ArrayList();

    // list holding tag names causing stop of recursive search
    // for properties
    private final static List HIERARCHY_STOP_TAGS = new ArrayList();

    public HibernateMappingPlugin(JellyTemplateEngine jellyTemplateEngine, MetadataProvider metadataProvider,
                                  WriterMapper writerMapper) {
        super(jellyTemplateEngine, metadataProvider, writerMapper);
        setFileregex("\\.java");
        setFilereplace("\\.hbm.xml");
        setMultioutput(true);

        new TagLibrary(metadataProvider);
    }

    /**
     * whether we sould generate given class. we generate if class contains hibernate.class
     * tag on it. class could be as well abstract, because real stuff lives in polymorphic
     * subclasses
     */
    public boolean shouldGenerate(Object metadata) {
        JavaClass clazz = (JavaClass) metadata;

        return clazz.getTagByName("hibernate.class") != null;
    }

    /**
     * provide combined list of all the tags with given tag names
     */
    public List getTags(AbstractJavaEntity metadata, Collection tagNames) {
        ArrayList al = new ArrayList();

        for (Iterator iter = tagNames.iterator(); iter.hasNext();) {
            al.addAll(Arrays.asList(metadata.getTagsByName((String) iter.next())));
        }

        return al;
    }

    /**
     * provide list of tags from pipe separated string
     */
    public List getTags(AbstractJavaEntity metadata, String tagNames) {
        return getTags(metadata, Arrays.asList(tagNames.split("\\|")));
    }

    public boolean hasAtLeastOne(AbstractJavaEntity metadata, Collection tags) {
        return getTags(metadata, tags).size() >= 1;
    }

    public boolean hasAtMostOne(AbstractJavaEntity metadata, Collection tags) {
        return getTags(metadata, tags).size() <= 1;
    }

    public boolean hasOnlyOne(AbstractJavaEntity metadata, Collection tags) {
        return getTags(metadata, tags).size() == 1;
    }

    /**
     * dispatch qtag to correct jelly script. cache results statically
     */
    public String dispatchTag(String tagName) {
        String retval = (String) tagDispatch.get(tagName);

        if (retval == null) {
            // rig up correct script  name from tag name
            // first, kill hibernate
            if (!tagName.startsWith(TAG_PREFIX)) {
                return null;
            }

            // split on dash and capitalize first chars
            String[] nameParts = tagName.substring(TAG_PREFIX.length()).split("-");
            StringBuffer scriptName = new StringBuffer(PACKAGE_PREFIX);

            for (int i = 0; i < nameParts.length; i++) {
                scriptName.append(Character.toUpperCase(nameParts[i].charAt(0)));
                scriptName.append(nameParts[i].substring(1));
            }

            scriptName.append(".jelly");
            retval = scriptName.toString();
            tagDispatch.put(tagName, retval);
        }

        return retval;
    }

    /**
     * provide list of subclasses for given class
     */
    public List getSubclasses(JavaClass clazz) {
        ArrayList result = new ArrayList();

        getSubclassesWithTagRecursive(result, clazz, "hibernate.subclass");
        Collections.sort(result);

        return result;
    }

    /**
     * provide list of joined subclasses for given class
     */
    public List getJoinedSubclasses(JavaClass clazz) {
        ArrayList result = new ArrayList();

        getSubclassesWithTagRecursive(result, clazz, "hibernate.joined-subclass");
        Collections.sort(result);

        return result;
    }

    /**
     * recursively retrieve list of derived classes with certain tag, cut search on found tag
     */
    private void getSubclassesWithTagRecursive(List found, JavaClass clazz, String tagName) {
        JavaClass[] subclasses = clazz.getDerivedClasses();

        for (int i = 0; i < subclasses.length; i++) {
            if (subclasses[i].getSuperJavaClass() == clazz) {
                if (subclasses[i].getTagByName(tagName) != null) {
                    found.add(subclasses[i]);
                } else {
                    getSubclassesWithTagRecursive(found, subclasses[i], tagName);
                }
            }
        }
    }

    /**
     * derive property name from metadata. strip / decapitalize
     */
    public String getPropertyName(AbstractJavaEntity metadata) {
        if (metadata instanceof JavaMethod) {
            return ((JavaMethod) metadata).getPropertyName();
        } else if (metadata instanceof JavaField) {
            return metadata.getName();
        }

        // maybe throw exception?
        return null;
    }

    /**
     * get property access type. "field" for fields, nothing else
     *
     * @deprecated
     */
    public String getPropertyAccess(AbstractJavaEntity metadata) {
        if (metadata instanceof JavaField) {
            return "field";
        }

        return null;
    }

    /**
     * provide list of hibernate properties for class. it could be getter, as well
     * as field ( for direct property access )
     */
    public List getClassProperties(JavaClass clazz) {
        ArrayList retval = new ArrayList();

        getPropertiesRecursive(clazz, null, retval);

        return retval;
    }

    /**
     * provide list of hibernate properties for subclass. it could be getter, as well
     * as field ( for direct property access ). we stop at hibernate.class
     */
    public List getSubclassProperties(JavaClass clazz) {
        ArrayList retval = new ArrayList();

        getPropertiesRecursive(clazz, HIERARCHY_STOP_TAGS, retval);

        return retval;
    }

    /**
     * gather hibernate propertis from given class into list
     */
    public void getProperties(JavaClass clazz, List accumulate) {
        int i;

        // walk through  property getters
        BeanProperty[] beanProperties = clazz.getBeanProperties();
        HibernateProperty property;
        
        for (i = 0; i < beanProperties.length; i++) {
            // property is ours, if we have at least one of designated property tags
            // and there is accessor
            if (beanProperties[i] != null && beanProperties[i].getAccessor() != null) {

                property = new HibernateProperty();
                property.setName(beanProperties[i].getName());
                property.setEntity(beanProperties[i].getAccessor());

                if (!accumulate.contains(property)) {
                    accumulate.add(property);
                }
            }
        }

        JavaField[] fields = clazz.getFields();

        for (i = 0; i < fields.length; i++) {
            if (!getTags(fields[i], getPropertyTagList()).isEmpty()) {
                property = new HibernateProperty();
                property.setName(fields[i].getName());
                property.setEntity(fields[i]);
                property.setAccess("field");

                if (!accumulate.contains(property)) {
                    accumulate.add(property);
                }
            }
        }
    }

    /**
     * recursive property retrival stopping at stop tag
     */
    void getPropertiesRecursive(JavaClass clazz, Collection stopTags, List accumulate) {
        getProperties(clazz, accumulate);

        JavaClass superclass = clazz.getSuperJavaClass();

        if (superclass != null) {
            // stop recursion?
            if (stopTags != null && !getTags(superclass, stopTags).isEmpty()) {
                // yep.
                return;
            }

            getPropertiesRecursive(superclass, stopTags, accumulate);
        }
    }

    /**
     * provide list of valid property tags
     */
    public List getPropertyTagList() {
        return PROPERTY_TAGS;

    }

    static {
        PROPERTY_TAGS.add("hibernate.property");
        PROPERTY_TAGS.add("hibernate.many-to-one");
        PROPERTY_TAGS.add("hibernate.one-to-one");
        PROPERTY_TAGS.add("hibernate.component");
        PROPERTY_TAGS.add("hibernate.dynamic-component");
        PROPERTY_TAGS.add("hibernate.any");
        PROPERTY_TAGS.add("hibernate.map");
        PROPERTY_TAGS.add("hibernate.set");
        PROPERTY_TAGS.add("hibernate.list");
        PROPERTY_TAGS.add("hibernate.bag");
        PROPERTY_TAGS.add("hibernate.idbag");
        PROPERTY_TAGS.add("hibernate.array");
        PROPERTY_TAGS.add("hibernate.primitive");
        PROPERTY_TAGS.add("hibernate.key-property");

        HIERARCHY_STOP_TAGS.add("hibernate.class");
        HIERARCHY_STOP_TAGS.add("hibernate.subclass");
        HIERARCHY_STOP_TAGS.add("hibernate.joined-subclass");

    }
}
