package org.xdoclet.plugin.web;

import com.thoughtworks.qdox.model.JavaClass;
import org.apache.commons.collections.Predicate;

/**
 * A Predicate that filters a JavaClass Collection with classes being of a
 * given class (or several), are concrete implementations (not abstract nor interfaces)
 * and which have a given tag.
 *
 *
 * <p><br/><br/>created     Sep 22, 2003 2:57:23 AM</p>
 * @author     <a href="mailto:greg_greg@freestart.hu">Grgory Joseph</a>
 * @author     $Author: moinsmoins $ (last edit)
 * @version    $Revision: 1.4 $
 */
public class ClassTagPredicate implements Predicate {
    private String[] classNames;
    private String tag;

    /**
     * Constructs a Predicate that will evaluate to true for each class
     * which <code>isA(className)</code> and has a <code>tag</code> tag.
     *
     * @param className
     * @param tag
     */
    public ClassTagPredicate(String className, String tag) {
        this.classNames = new String[]{className};
        this.tag = tag;
    }

    /**
     * Constructs a Predicate that will evaluate to true for each class
     * for which <code>isA(className)</code> evaluates to true for any of
     * the classNames array item and has a <code>tag</code> tag.
     *
     * @param classNames
     * @param tag
     */
    public ClassTagPredicate(String[] classNames, String tag) {
        this.classNames = classNames;
        this.tag = tag;
    }

    /**
     * Constructs a Predicate that will evaluate to true for each class
     * for which <code>isA(className)</code> evaluates to true for any of
     * the classNames array item.
     *
     * @param classNames
     */
    public ClassTagPredicate(String[] classNames) {
        this.classNames = classNames;
        this.tag = null;
    }

    public boolean evaluate(Object o) {
        JavaClass clazz = (JavaClass) o;

        int i = 0;
        boolean isA = false;
        while (isA == false && i < classNames.length) {
            isA = clazz.isA(classNames[i]);
            i++;
        }

        boolean hasTag;
        if (tag != null) {
            hasTag = clazz.getTagByName(tag) != null;
        } else {
            hasTag = true; // this is fake but means we ignore the predicated tag
        }

        boolean isConcrete = !(clazz.isAbstract() || clazz.isInterface());

        return isA && hasTag && isConcrete;
    }
}
